@@ -21,6 +21,7 @@
 #include <lttoolbox/xml_parse_util.h>
 #include <lttoolbox/string_to_wostream.h>
 
+#include <string>
 #include <cstdlib>
 #include <iostream>
 #include <libxml/encoding.h>
@@ -46,11 +47,11 @@ wstring const Compiler::COMPILER_S_ELEM             = L"s";
 wstring const Compiler::COMPILER_REGEXP_ELEM        = L"re";
 wstring const Compiler::COMPILER_SECTION_ELEM       = L"section";
 wstring const Compiler::COMPILER_ID_ATTR            = L"id";
-wstring const Compiler::COMPILER_TYPE_ATTR	    = L"type";
+wstring const Compiler::COMPILER_TYPE_ATTR          = L"type";
 wstring const Compiler::COMPILER_IDENTITY_ELEM      = L"i";
 wstring const Compiler::COMPILER_IDENTITYGROUP_ELEM = L"ig";
-wstring const Compiler::COMPILER_JOIN_ELEM	    = L"j";
-wstring const Compiler::COMPILER_BLANK_ELEM	    = L"b";
+wstring const Compiler::COMPILER_JOIN_ELEM          = L"j";
+wstring const Compiler::COMPILER_BLANK_ELEM         = L"b";
 wstring const Compiler::COMPILER_POSTGENERATOR_ELEM = L"a";
 wstring const Compiler::COMPILER_GROUP_ELEM         = L"g";
 wstring const Compiler::COMPILER_LEMMA_ATTR         = L"lm";
@@ -60,11 +61,13 @@ wstring const Compiler::COMPILER_ALT_ATTR           = L"alt";
 wstring const Compiler::COMPILER_V_ATTR             = L"v";
 wstring const Compiler::COMPILER_VL_ATTR            = L"vl";
 wstring const Compiler::COMPILER_VR_ATTR            = L"vr";
+wstring const Compiler::COMPILER_WEIGHT_ATTR        = L"w";
 
 Compiler::Compiler() :
 reader(0),
 verbose(false),
 first_element(false),
+default_weight(0.0000),
 acx_current_char(0)
 {
 }
@@ -74,14 +77,14 @@ Compiler::~Compiler()
 }
 
 void
-Compiler::parseACX(string const &fichero, wstring const &dir)
+Compiler::parseACX(string const &file, wstring const &dir)
 {
   if(dir == COMPILER_RESTRICTION_LR_VAL)
   {
-    reader = xmlReaderForFile(fichero.c_str(), NULL, 0);
+    reader = xmlReaderForFile(file.c_str(), NULL, 0);
     if(reader == NULL)
     {
-      wcerr << "Error: cannot open '" << fichero << "'." << endl;
+      wcerr << "Error: cannot open '" << file << "'." << endl;
       exit(EXIT_FAILURE);
     }
     int ret = xmlTextReaderRead(reader);
@@ -94,13 +97,13 @@ Compiler::parseACX(string const &fichero, wstring const &dir)
 }
 
 void
-Compiler::parse(string const &fichero, wstring const &dir)
+Compiler::parse(string const &file, wstring const &dir)
 {
   direction = dir;
-  reader = xmlReaderForFile(fichero.c_str(), NULL, 0);
+  reader = xmlReaderForFile(file.c_str(), NULL, 0);
   if(reader == NULL)
   {
-    wcerr << "Error: Cannot open '" << fichero << "'." << endl;
+    wcerr << "Error: Cannot open '" << file << "'." << endl;
     exit(EXIT_FAILURE);
   }
 
@@ -119,13 +122,11 @@ Compiler::parse(string const &fichero, wstring const &dir)
   xmlFreeTextReader(reader);
   xmlCleanupParser();
 
- 
-  // Minimize transducers 
-  for(map<wstring, Transducer, Ltstr>::iterator it = sections.begin(),
-                                               limit = sections.end(); 
-      it != limit; it++)
+
+  // Minimize transducers
+  for(auto& it : sections)
   {
-    (it->second).minimize();
+    it.second.minimize();
   }
 }
 
@@ -133,28 +134,28 @@ Compiler::parse(string const &fichero, wstring const &dir)
 void
 Compiler::procAlphabet()
 {
-  int tipo=xmlTextReaderNodeType(reader);
+  int type=xmlTextReaderNodeType(reader);
 
-  if(tipo != XML_READER_TYPE_END_ELEMENT)
+  if(type != XML_READER_TYPE_END_ELEMENT)
   {
     int ret = xmlTextReaderRead(reader);
     if(ret == 1)
-    {          
-      xmlChar const *valor = xmlTextReaderConstValue(reader);
-      letters = XMLParseUtil::towstring(valor);
-      bool espai = true;
-      for(unsigned int i = 0; i < letters.length(); i++)  
+    {
+      xmlChar const *value = xmlTextReaderConstValue(reader);
+      letters = XMLParseUtil::towstring(value);
+      bool space = true;
+      for(unsigned int i = 0; i < letters.length(); i++)
       {
-        if(!isspace(letters.at(i))) 
+        if(!isspace(letters.at(i)))
         {
-          espai = false;
+          space = false;
           break;
         }
       }
-      if(espai == true)  // libxml2 returns '\n' for <alphabet></alphabet>, should be empty
+      if(space == true)  // libxml2 returns '\n' for <alphabet></alphabet>, should be empty
       {
         letters = L"";
-      } 
+      }
     }
     else
     {
@@ -174,9 +175,9 @@ Compiler::procSDef()
 void
 Compiler::procParDef()
 {
-  int tipo=xmlTextReaderNodeType(reader);
+  int type=xmlTextReaderNodeType(reader);
 
-  if(tipo != XML_READER_TYPE_END_ELEMENT)
+  if(type != XML_READER_TYPE_END_ELEMENT)
   {
     current_paradigm = attrib(COMPILER_N_ATTR);
   }
@@ -192,31 +193,32 @@ Compiler::procParDef()
 }
 
 int
-Compiler::matchTransduction(list<int> const &pi, 
-				 list<int> const &pd, 
-				 int estado, Transducer &t)
+Compiler::matchTransduction(list<int> const &pi,
+                           list<int> const &pd,
+                           int state, Transducer &t,
+                           double const &entry_weight)
 {
-  list<int>::const_iterator izqda, dcha, limizqda, limdcha;
+  list<int>::const_iterator left, right, limleft, limright;
 
   if(direction == COMPILER_RESTRICTION_LR_VAL)
   {
-    izqda = pi.begin();
-    dcha = pd.begin();
-    limizqda = pi.end();
-    limdcha = pd.end();
+    left = pi.begin();
+    right = pd.begin();
+    limleft = pi.end();
+    limright = pd.end();
   }
   else
   {
-    izqda = pd.begin();
-    dcha = pi.begin();
-    limizqda = pd.end();
-    limdcha = pi.end();
+    left = pd.begin();
+    right = pi.begin();
+    limleft = pd.end();
+    limright = pi.end();
   }
- 
+
 
   if(pi.size() == 0 && pd.size() == 0)
   {
-    estado = t.insertNewSingleTransduction(alphabet(0, 0), estado);
+    state = t.insertNewSingleTransduction(alphabet(0, 0), state, default_weight);
   }
   else
   {
@@ -225,50 +227,60 @@ Compiler::matchTransduction(list<int> const &pi,
 
     while(true)
     {
-      int etiqueta;
-      
+      int tag;
+
       acx_map_ptr = acx_map.end();
-      
-      if(izqda == limizqda && dcha == limdcha)
+
+      if(left == limleft && right == limright)
       {
         break;
       }
-      else if(izqda == limizqda)
+      else if(left == limleft)
       {
-        etiqueta = alphabet(0, *dcha);
-        dcha++;
+        tag = alphabet(0, *right);
+        right++;
       }
-      else if(dcha == limdcha)
+      else if(right == limright)
       {
-        etiqueta = alphabet(*izqda, 0);
-        acx_map_ptr = acx_map.find(*izqda);
+        tag = alphabet(*left, 0);
+        acx_map_ptr = acx_map.find(*left);
         rsymbol = 0;
-        izqda++;
+        left++;
       }
       else
       {
-        etiqueta = alphabet(*izqda, *dcha);
-        acx_map_ptr = acx_map.find(*izqda);
-        rsymbol = *dcha;
-        izqda++;
-        dcha++;
+        tag = alphabet(*left, *right);
+        acx_map_ptr = acx_map.find(*left);
+        rsymbol = *right;
+        left++;
+        right++;
       }
 
-      int nuevo_estado = t.insertSingleTransduction(etiqueta, estado);
-      
+      double weight_value;
+
+      if(left == limleft && right == limright)
+      {
+        weight_value = entry_weight;
+      }
+      else
+      {
+        weight_value = default_weight;
+      }
+
+      int new_state = t.insertSingleTransduction(tag, state, weight_value);
+
       if(acx_map_ptr != acx_map.end())
       {
-        for(set<int>::iterator it = acx_map_ptr->second.begin(); 
-            it != acx_map_ptr->second.end(); it++)
-        { 
-          t.linkStates(estado, nuevo_estado, alphabet(*it ,rsymbol));
+        for(auto& it : acx_map_ptr->second)
+        {
+          t.linkStates(state, new_state, alphabet(it ,rsymbol), weight_value);
         }
       }
-      estado = nuevo_estado;
+      state = new_state;
     }
   }
 
-  return estado;
+  return state;
 }
 
 
@@ -283,21 +295,21 @@ Compiler::requireEmptyError(wstring const &name)
   }
 }
 
-bool 
+bool
 Compiler::allBlanks()
 {
   bool flag = true;
   wstring text = XMLParseUtil::towstring(xmlTextReaderConstValue(reader));
-  
-  for(unsigned int i = 0, limit = text.size(); i < limit; i++)
+
+  for(auto c : text)
   {
-    flag = flag && iswspace(text[i]);
+    flag = flag && iswspace(c);
   }
-  
+
   return flag;
 }
 
-void 
+void
 Compiler::readString(list<int> &result, wstring const &name)
 {
   if(name == L"#text")
@@ -325,8 +337,8 @@ Compiler::readString(list<int> &result, wstring const &name)
   }
   else if(name == COMPILER_GROUP_ELEM)
   {
-    int tipo=xmlTextReaderNodeType(reader);
-    if(tipo != XML_READER_TYPE_END_ELEMENT)
+    int type=xmlTextReaderNodeType(reader);
+    if(type != XML_READER_TYPE_END_ELEMENT)
     {
       result.push_back(static_cast<int>(L'#'));
     }
@@ -342,7 +354,7 @@ Compiler::readString(list<int> &result, wstring const &name)
       wcerr << L"): Undefined symbol '" << symbol << L"'." << endl;
       exit(EXIT_FAILURE);
     }
-    
+
     result.push_back(alphabet(symbol));
   }
   else
@@ -363,12 +375,12 @@ Compiler::skipBlanks(wstring &name)
     {
       if(!allBlanks())
       {
-        wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader); 
+        wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
         wcerr << L"): Invalid construction." << endl;
         exit(EXIT_FAILURE);
       }
     }
-    
+
     xmlTextReaderRead(reader);
     name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
   }
@@ -391,7 +403,7 @@ Compiler::skip(wstring &name, wstring const &elem, bool open)
   {
     slash = L"/";
   }
-  
+
   while(name == L"#text" || name == L"#comment")
   {
     if(name != L"#comment")
@@ -404,21 +416,22 @@ Compiler::skip(wstring &name, wstring const &elem, bool open)
       }
     }
     xmlTextReaderRead(reader);
-    name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));  
-  }    
-    
+    name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
+  }
+
   if(name != elem)
   {
     wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
     wcerr << L"): Expected '<" << slash << elem << L">'." << endl;
     exit(EXIT_FAILURE);
-  }  
+  }
 }
 
 EntryToken
-Compiler::procIdentity(bool ig)
+Compiler::procIdentity(wstring const &wsweight, bool ig)
 {
   list<int> both_sides;
+  double entry_weight = stod(wsweight);
 
   if(!xmlTextReaderIsEmptyElement(reader))
   {
@@ -435,34 +448,35 @@ Compiler::procIdentity(bool ig)
       readString(both_sides, name);
     }
   }
-  
-  if(verbose && first_element && (both_sides.front() == (int)L' ')) 
+
+  if(verbose && first_element && (both_sides.front() == (int)L' '))
   {
     wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
-    wcerr << L"): Entry begins with space." << endl; 
+    wcerr << L"): Entry begins with space." << endl;
   }
-  first_element = false; 
+  first_element = false;
   EntryToken e;
   if(ig)
   {
     list<int> right;
     right.push_back(static_cast<int>(L'#'));
     right.insert(right.end(), both_sides.begin(), both_sides.end());
-    e.setSingleTransduction(both_sides, right);
+    e.setSingleTransduction(both_sides, right, entry_weight);
   }
   else
   {
-    e.setSingleTransduction(both_sides, both_sides);
+    e.setSingleTransduction(both_sides, both_sides, entry_weight);
   }
   return e;
 }
 
 EntryToken
-Compiler::procTransduction()
+Compiler::procTransduction(wstring const &wsweight)
 {
   list<int> lhs, rhs;
+  double entry_weight = stod(wsweight);
   wstring name;
-  
+
   skip(name, COMPILER_LEFT_ELEM);
 
   if(!xmlTextReaderIsEmptyElement(reader))
@@ -480,13 +494,13 @@ Compiler::procTransduction()
     }
   }
 
-  if(verbose && first_element && (lhs.front() == (int)L' ')) 
+  if(verbose && first_element && (lhs.front() == (int)L' '))
   {
     wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
     wcerr << L"): Entry begins with space." << endl;
   }
   first_element = false;
- 
+
   skip(name, COMPILER_RIGHT_ELEM);
 
   if(!xmlTextReaderIsEmptyElement(reader))
@@ -501,13 +515,13 @@ Compiler::procTransduction()
         break;
       }
       readString(rhs, name);
-    }    
+    }
   }
 
   skip(name, COMPILER_PAIR_ELEM, false);
-  
+
   EntryToken e;
-  e.setSingleTransduction(lhs, rhs);
+  e.setSingleTransduction(lhs, rhs, entry_weight);
   return e;
 }
 
@@ -515,29 +529,29 @@ wstring
 Compiler::attrib(wstring const &name)
 {
   return XMLParseUtil::attrib(reader, name);
-} 
+}
 
 EntryToken
 Compiler::procPar()
 {
   EntryToken e;
-  wstring nomparadigma = attrib(COMPILER_N_ATTR);
+  wstring paradigm_name = attrib(COMPILER_N_ATTR);
   first_element = false;
 
-  if(current_paradigm != L"" && nomparadigma == current_paradigm)
+  if(current_paradigm != L"" && paradigm_name == current_paradigm)
   {
     wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
-    wcerr << L"): Paradigm refers to itself '" << nomparadigma << L"'." <<endl;
+    wcerr << L"): Paradigm refers to itself '" << paradigm_name << L"'." <<endl;
     exit(EXIT_FAILURE);
   }
 
-  if(paradigms.find(nomparadigma) == paradigms.end())
+  if(paradigms.find(paradigm_name) == paradigms.end())
   {
     wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
-    wcerr << L"): Undefined paradigm '" << nomparadigma << L"'." << endl;
+    wcerr << L"): Undefined paradigm '" << paradigm_name << L"'." << endl;
     exit(EXIT_FAILURE);
   }
-  e.setParadigm(nomparadigma);
+  e.setParadigm(paradigm_name);
   return e;
 }
 
@@ -549,24 +563,24 @@ Compiler::insertEntryTokens(vector<EntryToken> const &elements)
     // compilation of paradigms
     Transducer &t = paradigms[current_paradigm];
     int e = t.getInitial();
-    
-    for(unsigned int i = 0, limit = elements.size(); i < limit; i++)
+
+    for(auto& element : elements)
     {
-      if(elements[i].isParadigm())
+      if(element.isParadigm())
       {
-	e = t.insertTransducer(e, paradigms[elements[i].paradigmName()]);
+        e = t.insertTransducer(e, paradigms[element.paradigmName()]);
       }
-      else if(elements[i].isSingleTransduction())
+      else if(element.isSingleTransduction())
       {
-        e = matchTransduction(elements[i].left(), 
-                                  elements[i].right(), e, t);
+        e = matchTransduction(element.left(),
+                              element.right(), e, t, element.entryWeight());
       }
-      else if(elements[i].isRegexp())
+      else if(element.isRegexp())
       {
-	RegexpCompiler analyzer;
-	analyzer.initialize(&alphabet);
-	analyzer.compile(elements[i].regExp());
-	e = t.insertTransducer(e, analyzer.getTransducer(), alphabet(0,0));
+        RegexpCompiler analyzer;
+        analyzer.initialize(&alphabet);
+        analyzer.compile(element.regExp());
+        e = t.insertTransducer(e, analyzer.getTransducer(), alphabet(0,0));
       }
       else
       {
@@ -575,67 +589,67 @@ Compiler::insertEntryTokens(vector<EntryToken> const &elements)
         exit(EXIT_FAILURE);
       }
     }
-    t.setFinal(e);
+    t.setFinal(e, default_weight);
   }
   else
   {
-    // compilación de dictionary
+    // dictionary compilation
 
     Transducer &t = sections[current_section];
     int e = t.getInitial();
 
-    for(unsigned int i = 0, limit = elements.size(); i < limit; i++)
+    for(size_t i = 0, limit = elements.size(); i < limit; i++)
     {
       if(elements[i].isParadigm())
       {
         if(i == elements.size()-1)
-	{
-	  // paradigma sufijo
-	  if(suffix_paradigms[current_section].find(elements[i].paradigmName()) != suffix_paradigms[current_section].end())
-	  {
-	    t.linkStates(e, suffix_paradigms[current_section][elements[i].paradigmName()], 0);
+        {
+          // suffix paradigm
+          if(suffix_paradigms[current_section].find(elements[i].paradigmName()) != suffix_paradigms[current_section].end())
+          {
+            t.linkStates(e, suffix_paradigms[current_section][elements[i].paradigmName()], 0, elements[i].entryWeight());
             e = postsuffix_paradigms[current_section][elements[i].paradigmName()];
-	  }
+          }
           else
           {
-            e = t.insertNewSingleTransduction(alphabet(0, 0), e);
+            e = t.insertNewSingleTransduction(alphabet(0, 0), e, elements[i].entryWeight());
             suffix_paradigms[current_section][elements[i].paradigmName()] = e;
             e = t.insertTransducer(e, paradigms[elements[i].paradigmName()]);
             postsuffix_paradigms[current_section][elements[i].paradigmName()] = e;
-	  }
-	}
+          }
+        }
         else if(i == 0)
-	{
-          // paradigma prefijo
-	  if(prefix_paradigms[current_section].find(elements[i].paradigmName()) != prefix_paradigms[current_section].end())
-	  {
+        {
+          // prefix paradigm
+          if(prefix_paradigms[current_section].find(elements[i].paradigmName()) != prefix_paradigms[current_section].end())
+          {
             e = prefix_paradigms[current_section][elements[i].paradigmName()];
-	  }
-	  else
-	  {
+          }
+          else
+          {
             e = t.insertTransducer(e, paradigms[elements[i].paradigmName()]);
             prefix_paradigms[current_section][elements[i].paradigmName()] = e;
-	  }
+          }
         }
-	else
-	{
-          // paradigma intermedio
+        else
+        {
+          // intermediate paradigm
           e = t.insertTransducer(e, paradigms[elements[i].paradigmName()]);
-	}
+        }
       }
       else if(elements[i].isRegexp())
       {
-	RegexpCompiler analyzer;
-	analyzer.initialize(&alphabet);
-	analyzer.compile(elements[i].regExp());
-	e = t.insertTransducer(e, analyzer.getTransducer(), alphabet(0,0));
+        RegexpCompiler analyzer;
+        analyzer.initialize(&alphabet);
+        analyzer.compile(elements[i].regExp());
+        e = t.insertTransducer(e, analyzer.getTransducer(), alphabet(0,0));
       }
       else
       {
-        e = matchTransduction(elements[i].left(), elements[i].right(), e, t);
+        e = matchTransduction(elements[i].left(), elements[i].right(), e, t, elements[i].entryWeight());
       }
     }
-    t.setFinal(e);
+    t.setFinal(e, default_weight);
   }
 }
 
@@ -646,27 +660,27 @@ Compiler::requireAttribute(wstring const &value, wstring const &attrname,
 {
   if(value == L"")
   {
-    wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);  
+    wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
     wcerr << L"): '<" << elemname;
     wcerr << L"' element must specify non-void '";
     wcerr << attrname << L"' attribute." << endl;
     exit(EXIT_FAILURE);
-  }  
+  }
 }
 
 
 void
 Compiler::procSection()
 {
-  int tipo=xmlTextReaderNodeType(reader);
+  int type=xmlTextReaderNodeType(reader);
 
-  if(tipo != XML_READER_TYPE_END_ELEMENT)
+  if(type != XML_READER_TYPE_END_ELEMENT)
   {
     wstring const &id = attrib(COMPILER_ID_ATTR);
     wstring const &type = attrib(COMPILER_TYPE_ATTR);
     requireAttribute(id, COMPILER_ID_ATTR, COMPILER_SECTION_ELEM);
     requireAttribute(type, COMPILER_TYPE_ATTR, COMPILER_SECTION_ELEM);
-    
+
     current_section = id;
     current_section += L"@";
     current_section.append(type);
@@ -680,15 +694,16 @@ Compiler::procSection()
 void
 Compiler::procEntry()
 {
-  wstring atributo=this->attrib(COMPILER_RESTRICTION_ATTR);
-  wstring ignore = this->attrib(COMPILER_IGNORE_ATTR);
-  wstring altval = this->attrib(COMPILER_ALT_ATTR);
-  wstring varval = this->attrib(COMPILER_V_ATTR);
-  wstring varl   = this->attrib(COMPILER_VL_ATTR);
-  wstring varr   = this->attrib(COMPILER_VR_ATTR);
+  wstring attribute = this->attrib(COMPILER_RESTRICTION_ATTR);
+  wstring ignore    = this->attrib(COMPILER_IGNORE_ATTR);
+  wstring altval    = this->attrib(COMPILER_ALT_ATTR);
+  wstring varval    = this->attrib(COMPILER_V_ATTR);
+  wstring varl      = this->attrib(COMPILER_VL_ATTR);
+  wstring varr      = this->attrib(COMPILER_VR_ATTR);
+  wstring wsweight  = this->attrib(COMPILER_WEIGHT_ATTR);
 
   // if entry is masked by a restriction of direction or an ignore mark
-  if((atributo != L"" && atributo != direction) 
+  if((attribute != L"" && attribute != direction)
    || ignore == COMPILER_IGNORE_YES_VAL
    || (altval != L"" && altval != alt)
    || (direction == COMPILER_RESTRICTION_RL_VAL && varval != L"" && varval != variant)
@@ -707,6 +722,11 @@ Compiler::procEntry()
     return;
   }
 
+  if(wsweight == L"")
+  {
+    wsweight = L"0.0000";
+  }
+
   vector<EntryToken> elements;
 
   while(true)
@@ -726,18 +746,18 @@ Compiler::procEntry()
       first_element = true;
     }
 
-    int tipo = xmlTextReaderNodeType(reader);
+    int type = xmlTextReaderNodeType(reader);
     if(name == COMPILER_PAIR_ELEM)
-    {      
-      elements.push_back(procTransduction());
+    {
+      elements.push_back(procTransduction(wsweight));
     }
     else if(name == COMPILER_IDENTITY_ELEM)
     {
-      elements.push_back(procIdentity(false));
+      elements.push_back(procIdentity(wsweight, false));
     }
     else if(name == COMPILER_IDENTITYGROUP_ELEM)
     {
-      elements.push_back(procIdentity(true));
+      elements.push_back(procIdentity(wsweight, true));
     }
     else if(name == COMPILER_REGEXP_ELEM)
     {
@@ -747,7 +767,7 @@ Compiler::procEntry()
     {
       elements.push_back(procPar());
 
-      // detección del uso de paradigmas no definidos
+      // detection of the use of undefined paradigms
 
       wstring const &p = elements.rbegin()->paradigmName();
 
@@ -757,22 +777,21 @@ Compiler::procEntry()
         wcerr << L"): Undefined paradigm '" << p << L"'." <<endl;
         exit(EXIT_FAILURE);
       }
-      // descartar entradas con paradigms vacíos (por las direciones,
-      // normalmente
+      // discard entries with empty paradigms (by the directions, normally)
       if(paradigms[p].isEmpty())
       {
-        while(name != COMPILER_ENTRY_ELEM || tipo != XML_READER_TYPE_END_ELEMENT)
+        while(name != COMPILER_ENTRY_ELEM || type != XML_READER_TYPE_END_ELEMENT)
         {
           xmlTextReaderRead(reader);
           name = XMLParseUtil::towstring(xmlTextReaderConstName(reader));
-          tipo = xmlTextReaderNodeType(reader);
+          type = xmlTextReaderNodeType(reader);
         }
         return;
       }
     }
-    else if(name == COMPILER_ENTRY_ELEM && tipo == XML_READER_TYPE_END_ELEMENT)
+    else if(name == COMPILER_ENTRY_ELEM && type == XML_READER_TYPE_END_ELEMENT)
     {
-      // insertar elements into letter transducer
+      // insert elements into letter transducer
       insertEntryTokens(elements);
       return;
     }
@@ -783,7 +802,7 @@ Compiler::procEntry()
     {
       wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
       wcerr << L"): Invalid inclusion of '<" << name << L">' into '<" << COMPILER_ENTRY_ELEM;
-      wcerr << L">'." << endl; 
+      wcerr << L">'." << endl;
       exit(EXIT_FAILURE);
     }
   }
@@ -792,32 +811,32 @@ Compiler::procEntry()
 void
 Compiler::procNodeACX()
 {
-  xmlChar  const *xnombre = xmlTextReaderConstName(reader);
-  wstring nombre = XMLParseUtil::towstring(xnombre);
-  if(nombre == L"#text")
+  xmlChar  const *xname = xmlTextReaderConstName(reader);
+  wstring name = XMLParseUtil::towstring(xname);
+  if(name == L"#text")
   {
     /* ignore */
   }
-  else if(nombre == L"analysis-chars")
+  else if(name == L"analysis-chars")
   {
     /* ignore */
   }
-  else if(nombre == L"char")
+  else if(name == L"char")
   {
     acx_current_char = static_cast<int>(attrib(L"value")[0]);
   }
-  else if(nombre == L"equiv-char")
+  else if(name == L"equiv-char")
   {
     acx_map[acx_current_char].insert(static_cast<int>(attrib(L"value")[0]));
   }
-  else if(nombre == L"#comment")
+  else if(name == L"#comment")
   {
     /* ignore */
   }
   else
   {
     wcerr << L"Error in ACX file (" << xmlTextReaderGetParserLineNumber(reader);
-    wcerr << L"): Invalid node '<" << nombre << L">'." << endl;
+    wcerr << L"): Invalid node '<" << name << L">'." << endl;
     exit(EXIT_FAILURE);
   }
 }
@@ -825,55 +844,55 @@ Compiler::procNodeACX()
 void
 Compiler::procNode()
 {
-  xmlChar const *xnombre = xmlTextReaderConstName(reader);
-  wstring nombre = XMLParseUtil::towstring(xnombre);
+  xmlChar const *xname = xmlTextReaderConstName(reader);
+  wstring name = XMLParseUtil::towstring(xname);
 
-  // HACER: optimizar el orden de ejecución de esta ristra de "ifs"
+  // TODO: optimize the execution order of the string "ifs"
 
-  if(nombre == L"#text")
+  if(name == L"#text")
   {
-    /* ignorar */
+    /* ignore */
   }
-  else if(nombre == COMPILER_DICTIONARY_ELEM)
+  else if(name == COMPILER_DICTIONARY_ELEM)
   {
-    /* ignorar */
+    /* ignore */
   }
-  else if(nombre == COMPILER_ALPHABET_ELEM)
+  else if(name == COMPILER_ALPHABET_ELEM)
   {
     procAlphabet();
   }
-  else if(nombre == COMPILER_SDEFS_ELEM)
+  else if(name == COMPILER_SDEFS_ELEM)
   {
-    /* ignorar */
+    /* ignore */
   }
-  else if(nombre == COMPILER_SDEF_ELEM)
+  else if(name == COMPILER_SDEF_ELEM)
   {
     procSDef();
   }
-  else if(nombre == COMPILER_PARDEFS_ELEM)
+  else if(name == COMPILER_PARDEFS_ELEM)
   {
-    /* ignorar */
+    /* ignore */
   }
-  else if(nombre == COMPILER_PARDEF_ELEM)
+  else if(name == COMPILER_PARDEF_ELEM)
   {
     procParDef();
   }
-  else if(nombre == COMPILER_ENTRY_ELEM)
+  else if(name == COMPILER_ENTRY_ELEM)
   {
     procEntry();
   }
-  else if(nombre == COMPILER_SECTION_ELEM)
+  else if(name == COMPILER_SECTION_ELEM)
   {
     procSection();
   }
-  else if(nombre== L"#comment")
+  else if(name== L"#comment")
   {
-    /* ignorar */
+    /* ignore */
   }
   else
   {
     wcerr << L"Error (" << xmlTextReaderGetParserLineNumber(reader);
-    wcerr << L"): Invalid node '<" << nombre << L">'." << endl;
+    wcerr << L"): Invalid node '<" << name << L">'." << endl;
     exit(EXIT_FAILURE);
   }
 }
@@ -889,28 +908,30 @@ Compiler::procRegexp()
   return et;
 }
 
-void 
+void
 Compiler::write(FILE *output)
 {
+  fwrite(HEADER_LTTOOLBOX, 1, 4, output);
+  uint64_t features = 0;
+  write_le(output, features);
+
   // letters
   Compression::wstring_write(letters, output);
-  
+
   // symbols
   alphabet.write(output);
-  
+
   // transducers
   Compression::multibyte_write(sections.size(), output);
 
-  int conta=0;
-  for(map<wstring, Transducer, Ltstr>::iterator it = sections.begin(),
-                                               limit = sections.end(); 
-      it != limit; it++)
+  int count=0;
+  for(auto& it : sections)
   {
-    conta++;
-    wcout << it->first << " " << it->second.size();
-    wcout << " " << it->second.numberOfTransitions() << endl;
-    Compression::wstring_write(it->first, output);
-    it->second.write(output);
+    count++;
+    wcout << it.first << " " << it.second.size();
+    wcout << " " << it.second.numberOfTransitions() << endl;
+    Compression::wstring_write(it.first, output);
+    it.second.write(output);
   }
 }
 
