@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005 Universitat d'Alacant / Universidad de Alicante
+ * Copyright (C) 2005-2019 Universitat d'Alacant / Universidad de Alicante
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -69,29 +69,34 @@ private:
   State initial_state;
 
   /**
-   * Set of final states of inconditional sections in the dictionaries
+   * Default value of weight unless specified
    */
-  set<Node *> inconditional;
+  double default_weight;
 
   /**
-   * Set of final states of standard sections in the dictionaries
+   * The final states of inconditional sections in the dictionaries
    */
-  set<Node *> standard;
+  map<Node *, double> inconditional;
 
   /**
-   * Set of final states of postblank sections in the dictionaries
+   * The final states of standard sections in the dictionaries
    */
-  set<Node *> postblank;
+  map<Node *, double> standard;
 
   /**
-   * Set of final states of preblank sections in the dictionaries
+   * The final states of postblank sections in the dictionaries
    */
-  set<Node *> preblank;
+  map<Node *, double> postblank;
+
+  /**
+   * The final states of preblank sections in the dictionaries
+   */
+  map<Node *, double> preblank;
 
   /**
    * Merge of 'inconditional', 'standard', 'postblank' and 'preblank' sets
    */
-  set<Node *> all_finals;
+  map<Node *, double> all_finals;
 
   /**
    * Queue of blanks, used in reading methods
@@ -186,6 +191,11 @@ private:
    */
   bool useDefaultIgnoredChars;
 
+  /**
+   * if true, displays the final weights (if any)
+   */
+  bool displayWeightsMode;
+
   /**
    * try analysing unknown words as compounds
    */
@@ -212,6 +222,16 @@ private:
    */
   int compound_max_elements;
 
+  /**
+   * Output no more than 'N' number of weighted analyses
+   */
+  int maxAnalyses;
+
+  /**
+   * Output no more than 'N' best weight classes
+   */
+  int maxWeightClasses;
+
   /**
    * Prints an error of input stream and exits
    */
@@ -247,14 +267,14 @@ private:
   bool isEscaped(wchar_t const c) const;
 
   /**
-   * Read text from stream (analysis version, also used in postgeneration)
+   * Read text from stream (analysis version)
    * @param input the stream to read
    * @return the next symbol in the stream
    */
   int readAnalysis(FILE *input);
 
   /**
-   * Read text from stream (generation version, also used in generation)
+   * Read text from stream (decomposition version)
    * @param input the stream to read
    * @return the next symbol in the stream
    */
@@ -311,6 +331,16 @@ private:
    */
   void writeEscaped(wstring const &str, FILE *output);
 
+  /**
+   * Write a string to an output stream.
+   * If we print a space, we may pop a space from blankqueue
+   * immediately, otherwise it should be printed afterwards.
+   *
+   * @param str the string to write, escaping characters
+   * @param output the stream to write in
+   * @return how many blanks to pop and print after printing lu
+   */
+  size_t writeEscapedPopBlanks(wstring const &str, FILE *output);
 
   /**
    * Write a string to an output stream, escaping all escapable characters
@@ -337,6 +367,16 @@ private:
    */
   void printWord(wstring const &sf, wstring const &lf, FILE *output);
 
+  /**
+   * Prints a word.
+   * If we print a space, we may pop a space from blankqueue.
+   *
+   * @param sf surface form of the word
+   * @param lf lexical form of the word
+   * @param output stream where the word is written
+   */
+  void printWordPopBlank(wstring const &sf, wstring const &lf, FILE *output);
+
   /**
    * Prints a word (Bilingual version)
    * @param sf surface form of the word
@@ -366,17 +406,27 @@ private:
   int readTMAnalysis(FILE *input);
 
   unsigned int lastBlank(wstring const &str);
+
+  /**
+   * Print one blankqueue item if there is one, or a given "space" value.
+   *
+   * @param val the space character to use if no blank queue
+   * @param output stream where the word is written
+   */
   void printSpace(wchar_t const val, FILE *output);
+
   void skipUntil(FILE *input, FILE *output, wint_t const character);
   static wstring removeTags(wstring const &str);
   wstring compoundAnalysis(wstring str, bool uppercase, bool firstupper);
   size_t firstNotAlpha(wstring const &sf);
 
   void analysis_wrapper_null_flush(FILE *input, FILE *output);
+  void lsx_wrapper_null_flush(FILE *input, FILE *output);
   void bilingual_wrapper_null_flush(FILE *input, FILE *output);
   void generation_wrapper_null_flush(FILE *input, FILE *output,
                                      GenerationMode mode);
   void postgeneration_wrapper_null_flush(FILE *input, FILE *output);
+  void intergeneration_wrapper_null_flush(FILE *input, FILE *output);
   void transliteration_wrapper_null_flush(FILE *input, FILE *output);
 
   wstring compose(wstring const &lexforms, wstring const &queue) const;
@@ -403,16 +453,16 @@ public:
   void tm_analysis(FILE *input = stdin, FILE *output = stdout);
   void generation(FILE *input = stdin, FILE *output = stdout, GenerationMode mode = gm_unknown);
   void postgeneration(FILE *input = stdin, FILE *output = stdout);
+  void intergeneration(FILE *input = stdin, FILE *output = stdout);
   void transliteration(FILE *input = stdin, FILE *output = stdout);
-  void decomposition(FILE *input = stdin, FILE *output = stdout);
   wstring biltrans(wstring const &input_word, bool with_delim = true);
   wstring biltransfull(wstring const &input_word, bool with_delim = true);
   void bilingual(FILE *input = stdin, FILE *output = stdout);
   pair<wstring, int> biltransWithQueue(wstring const &input_word, bool with_delim = true);
   wstring biltransWithoutQueue(wstring const &input_word, bool with_delim = true);
   void SAO(FILE *input = stdin, FILE *output = stdout);
-  void parseICX(string const &fichero);
-  void parseRCX(string const &fichero);
+  void parseICX(string const &file);
+  void parseRCX(string const &file);
 
   void load(FILE *input);
 
@@ -426,7 +476,10 @@ public:
   void setIgnoredChars(bool const value);
   void setRestoreChars(bool const value);
   void setNullFlush(bool const value);
-  void setUseDefaultIgnoredChars(bool);
+  void setUseDefaultIgnoredChars(bool const value);
+  void setDisplayWeightsMode(bool const value);
+  void setMaxAnalysesValue(int const value);
+  void setMaxWeightClassesValue(int const value);
   bool getNullFlush();
   bool getDecompoundingMode();
 };
