@@ -67,33 +67,31 @@ Transducer::operator =(Transducer const &t)
 }
 
 int
-Transducer::insertSingleTransduction(int const tag, int const source)
+Transducer::insertSingleTransduction(int const tag, int const source, double const weight)
 {
   if(transitions.find(source) != transitions.end())
   {
     if(transitions[source].count(tag) == 1)
     {
-      pair<multimap<int,int>::iterator, multimap<int,int>::iterator > range;
-      range = transitions[source].equal_range(tag);
-      return range.first->second;
+      auto range = transitions[source].equal_range(tag);
+      return range.first->second.first;
     }
     else if(transitions[source].count(tag) == 0)
     {
       // new state
       int state = newState();
-      transitions[source].insert(pair<int, int>(tag, state));
+      transitions[source].insert(make_pair(tag, make_pair(state, weight)));
       return state;
     }
     else if(transitions[source].count(tag) == 2)
     {
       // there's a local cycle, must be ignored and treated like in '1'
-      pair<multimap<int,int>::iterator, multimap<int,int>::iterator> range;
-      range = transitions[source].equal_range(tag);
+      auto range = transitions[source].equal_range(tag);
       for(; range.first != range.second; range.first++)
       {
-        if(range.first->second != source)
+        if(range.first->second.first != source)
         {
-          return range.first->second;
+          return range.first->second.first;
         }
       }
       return -1;
@@ -110,70 +108,66 @@ Transducer::insertSingleTransduction(int const tag, int const source)
 }
 
 int
-Transducer::insertNewSingleTransduction(int const tag, int const source)
+Transducer::insertNewSingleTransduction(int const tag, int const source, double const weight)
 {
   int state = newState();
-  transitions[source].insert(pair<int, int>(tag, state));
+  transitions[source].insert(make_pair(tag, make_pair(state, weight)));
   return state;
 }
 
 int
 Transducer::insertTransducer(int const source, Transducer &t,
-                                 int const epsilon_tag)
+                            int const epsilon_tag)
 {
-  map<int, int> relacion;
+  map<int, int> relation;
 
   t.joinFinals(epsilon_tag);
 
-  for(map<int, multimap<int, int> >::const_iterator it = t.transitions.begin(),
-                                                    limit = t.transitions.end();
-      it != limit; it++)
+  for(auto& it : t.transitions)
   {
-    relacion[it->first] = newState();
+    relation[it.first] = newState();
   }
 
-  for(map<int, multimap<int, int> >::const_iterator it = t.transitions.begin();
-      it != t.transitions.end(); it++)
+  for(auto& it : t.transitions)
   {
-    for(multimap<int, int>::const_iterator it2 = it->second.begin(),
-                                           limit2 = (it->second).end();
-        it2 != limit2; it2++)
+    for(auto& it2 : it.second)
     {
-      transitions[relacion[it->first]].insert(pair<int, int>(it2->first, relacion[it2->second]));
+      transitions[relation[it.first]].insert(make_pair(it2.first,
+                                                        make_pair(relation[it2.second.first],
+                                                                  it2.second.second)));
     }
   }
 
-  transitions[source].insert(pair<int, int>(epsilon_tag,
-					     relacion[t.initial]));
+  transitions[source].insert(make_pair(epsilon_tag,
+                                       make_pair(relation[t.initial], default_weight)));
 
-  return relacion[*(t.finals.begin())];
+  return relation[t.finals.begin()->first];
 }
 
 void
-Transducer::linkStates(int const source, int const destino,
-			    int const etiqueta)
+Transducer::linkStates(int const source, int const target,
+                       int const tag, double const weight)
 {
 
   if(transitions.find(source) != transitions.end() &&
-     transitions.find(destino) != transitions.end())
+     transitions.find(target) != transitions.end())
   {
     // new code
-    pair<multimap<int, int>::iterator, multimap<int, int>::iterator> range;
-    range = transitions[source].equal_range(etiqueta);
+    auto range = transitions[source].equal_range(tag);
     for(;range.first != range.second; range.first++)
     {
-      if(range.first->first == etiqueta && range.first->second == destino)
+      if(range.first->first == tag && range.first->second.first == target)
       {
         return;
       }
     }
     // end of new code
-    transitions[source].insert(pair<int, int>(etiqueta, destino));
+    transitions[source].insert(make_pair(tag, make_pair(target, weight)));
   }
   else
   {
     wcerr << L"Error: Trying to link nonexistent states (" << source;
-    wcerr << L", " << destino << L", " << etiqueta << L")" << endl;
+    wcerr << L", " << target << L", " << tag << L")" << endl;
     exit(EXIT_FAILURE);
   }
 }
@@ -185,7 +179,7 @@ Transducer::isFinal(int const state) const
 }
 
 void
-Transducer::setFinal(int const state, bool valor)
+Transducer::setFinal(int const state, double const weight, bool value)
 {
 /*
   int initial_copy = getInitial();
@@ -194,9 +188,9 @@ Transducer::setFinal(int const state, bool valor)
     wcerr << L"Setting initial state to final" << endl;
   }
 */
-  if(valor)
+  if(value)
   {
-    finals.insert(state);
+    finals.insert(make_pair(state, weight));
   }
   else
   {
@@ -221,16 +215,15 @@ Transducer::closure(int const state, int const epsilon_tag)
   while(nonvisited.size() > 0)
   {
     int auxest = *nonvisited.begin();
-    pair<multimap<int, int>::iterator, multimap<int, int>::iterator> rango;
-    rango = transitions[auxest].equal_range(epsilon_tag);
-    while(rango.first != rango.second)
+    auto range = transitions[auxest].equal_range(epsilon_tag);
+    while(range.first != range.second)
     {
-      if(result.find(rango.first->second) == result.end())
+      if(result.find(range.first->second.first) == result.end())
       {
-        result.insert(rango.first->second);
-        nonvisited.insert(rango.first->second);
+        result.insert(range.first->second.first);
+        nonvisited.insert(range.first->second.first);
       }
-      rango.first++;
+      range.first++;
     }
     nonvisited.erase(auxest);
   }
@@ -245,14 +238,13 @@ Transducer::joinFinals(int const epsilon_tag)
   {
     int state = newState();
 
-    for(set<int>::iterator it = finals.begin(), limit = finals.end();
-        it != limit; it++)
+    for(auto& it : finals)
     {
-      linkStates(*it, state, epsilon_tag);
+      linkStates(it.first, state, epsilon_tag, it.second);
     }
 
     finals.clear();
-    finals.insert(state);
+    finals.insert(make_pair(state, default_weight));
   }
   else if(finals.size() == 0)
   {
@@ -267,19 +259,19 @@ Transducer::isEmptyIntersection(set<int> const &s1, set<int> const &s2)
 
   if(s1.size() < s2.size())
   {
-    for(set<int>::const_iterator it = s1.begin(), limit = s1.end(); it != limit; it++)
+    for(auto& it : s1)
     {
-      if(s2.find(*it) != s2.end())
+      if(s2.count(it))
       {
-	return false;
+        return false;
       }
     }
   }
   else
   {
-    for(set<int>::const_iterator it = s2.begin(), limit = s2.end(); it != limit; it++)
+    for(auto& it : s2)
     {
-      if(s1.find(*it) != s1.end())
+      if(s1.count(it))
       {
         return false;
       }
@@ -293,85 +285,89 @@ void
 Transducer::determinize(int const epsilon_tag)
 {
   vector<set<int> > R(2);
-  map<int, set<int> > Q_prima;
-  map<set<int>, int> Q_prima_inv;
+  map<int, set<int> > Q_prime;
+  map<set<int>, int> Q_prime_inv;
 
-  map<int, multimap<int, int> > transitions_prima;
+  map<int, multimap<int, pair<int, double> > > transitions_prime;
 
-  unsigned int talla_Q_prima = 0;
-  Q_prima[0] = closure(initial, epsilon_tag);
+  unsigned int size_Q_prime = 0;
+  Q_prime[0] = closure(initial, epsilon_tag);
 
-  Q_prima_inv[Q_prima[0]] = 0;
+  Q_prime_inv[Q_prime[0]] = 0;
   R[0].insert(0);
 
-  int initial_prima = 0;
-  set<int> finals_prima;
+  int initial_prime = 0;
+  map<int, double> finals_prime;
 
   if(isFinal(initial))
   {
-    finals_prima.insert(0);
+    finals_prime.insert(make_pair(0, default_weight));
   }
 
   int t = 0;
 
-  while(talla_Q_prima != Q_prima.size())
+  while(size_Q_prime != Q_prime.size())
   {
-    talla_Q_prima = Q_prima.size();
+    size_Q_prime = Q_prime.size();
     R[(t+1)%2].clear();
 
-    for(set<int>::iterator it = R[t].begin(), limit = R[t].end();
-        it != limit; it++)
+    for(auto& it : R[t])
     {
-      if(!isEmptyIntersection(Q_prima[*it], finals))
+      set<int> finals_state;
+      for(auto& it2 : finals)
       {
-	finals_prima.insert(*it);
+        finals_state.insert(it2.first);
+      }
+      if(!isEmptyIntersection(Q_prime[it], finals_state))
+      {
+        double w = default_weight;
+        auto it3 = finals.find(it);
+        if (it3 != finals.end()) {
+          w = it3->second;
+        }
+        finals_prime.insert(make_pair(it, w));
       }
 
-      map<int, set<int> > mymap;
+      map<pair<int, double>, set<int> > mymap;
 
-      for(set<int>::iterator it2 = Q_prima[*it].begin(),
-                             limit2 = Q_prima[*it].end();
-	  it2 != limit2; it2++)
+      for(auto& it2 : Q_prime[it])
       {
-        for(multimap<int, int>::iterator it3 = transitions[*it2].begin(),
-                                         limit3 = transitions[*it2].end();
-	    it3 != limit3; it3++)
-	{
-	  if(it3->first != epsilon_tag)
-	  {
-	    set<int> c = closure(it3->second, epsilon_tag);
+        for(auto& it3 : transitions[it2])
+        {
+          if(it3.first != epsilon_tag)
+          {
+            auto c = closure(it3.second.first, epsilon_tag);
 
-	    for(set<int>::iterator it4 = c.begin(), limit4 = c.end();
-	        it4 != limit4; it4++)
-	    {
-	      mymap[it3->first].insert(*it4);
-	    }
-	  }
-	}
+            for(auto& it4 : c)
+            {
+              mymap[make_pair(it3.first, it3.second.second)].insert(it4);
+            }
+          }
+        }
       }
 
       // adding new states
-      for(map<int, set<int> >::iterator it2 = mymap.begin(), limit2 = mymap.end();
-	  it2 != limit2; it2++)
+      for(auto& it2 : mymap)
       {
-	if(Q_prima_inv.find(it2->second) == Q_prima_inv.end())
-	{
-	  int etiq = Q_prima.size();
-	  Q_prima[etiq] = it2->second;
-	  Q_prima_inv[it2->second] = etiq;
-	  R[(t+1)%2].insert(Q_prima_inv[it2->second]);
-          transitions_prima[etiq].clear();
-	}
-        transitions_prima[*it].insert(pair<int, int>(it2->first, Q_prima_inv[it2->second]));
+        if(Q_prime_inv.find(it2.second) == Q_prime_inv.end())
+        {
+          int tag = Q_prime.size();
+          Q_prime[tag] = it2.second;
+          Q_prime_inv[it2.second] = tag;
+          R[(t+1)%2].insert(Q_prime_inv[it2.second]);
+          transitions_prime[tag].clear();
+        }
+        transitions_prime[it].insert(make_pair(it2.first.first,
+                                                make_pair(Q_prime_inv[it2.second], it2.first.second)));
       }
     }
 
     t = (t+1)%2;
   }
 
-  transitions = transitions_prima;
-  finals = finals_prima;
-  initial = initial_prima;
+  transitions = transitions_prime;
+  finals = finals_prime;
+  initial = initial_prime;
 }
 
 
@@ -389,14 +385,14 @@ Transducer::optional(int const epsilon_tag)
 {
   joinFinals(epsilon_tag);
   int state = newState();
-  linkStates(state, initial, epsilon_tag);
+  linkStates(state, initial, epsilon_tag, default_weight);
   initial = state;
 
   state = newState();
-  linkStates(*finals.begin(), state, epsilon_tag);
+  linkStates(finals.begin()->first, state, epsilon_tag, finals.begin()->second);
   finals.clear();
-  finals.insert(state);
-  linkStates(initial, state, epsilon_tag);
+  finals.insert(make_pair(state, default_weight));
+  linkStates(initial, state, epsilon_tag, default_weight);
 }
 
 void
@@ -404,14 +400,14 @@ Transducer::oneOrMore(int const epsilon_tag)
 {
   joinFinals(epsilon_tag);
   int state = newState();
-  linkStates(state, initial, epsilon_tag);
+  linkStates(state, initial, epsilon_tag, default_weight);
   initial = state;
 
   state = newState();
-  linkStates(*finals.begin(), state, epsilon_tag);
+  linkStates(finals.begin()->first, state, epsilon_tag, finals.begin()->second);
   finals.clear();
-  finals.insert(state);
-  linkStates(state, initial, epsilon_tag);
+  finals.insert(make_pair(state, default_weight));
+  linkStates(state, initial, epsilon_tag, default_weight);
 }
 
 void
@@ -441,16 +437,16 @@ Transducer::hasNoFinals() const
   return finals.size() == 0;
 }
 
-map<int, multimap<int, int> >&
+map<int, multimap<int, pair<int, double> > >&
 Transducer::getTransitions()
 {
   return transitions;
 }
 
-set<int>
+map<int, double>
 Transducer::getFinals() const
 {
-  return set<int>(finals);
+  return map<int, double>(finals);
 }
 
 int
@@ -464,11 +460,9 @@ Transducer::numberOfTransitions() const
 {
   int counter = 0;
 
-  for(map<int, multimap<int, int> >::const_iterator it = transitions.begin(),
-                                                    limit = transitions.end();
-      it != limit; it++)
+  for(auto& it : transitions)
   {
-    counter += (it->second).size();
+    counter += it.second.size();
   }
 
   return counter;
@@ -477,12 +471,11 @@ Transducer::numberOfTransitions() const
 bool
 Transducer::isEmpty(int const state) const
 {
-  map<int, multimap<int, int> >::const_iterator it;
 
-  it = transitions.find(state);
+  auto it = transitions.find(state);
   if(it != transitions.end())
   {
-    if((it->second).size() > 0)
+    if(it->second.size() > 0)
     {
       return false;
     }
@@ -491,42 +484,72 @@ Transducer::isEmpty(int const state) const
   return true;
 }
 
+// Determine whether any weights are non-default (0)
+bool Transducer::weighted() {
+  for (auto& it : finals) {
+    if (it.second != default_weight) {
+      return true;
+    }
+  }
+  for (auto& it : transitions) {
+    for (auto& it2 : it.second) {
+      if (it2.second.second != default_weight) {
+        return true;
+      }
+    }
+  }
+  return false;
+}
+
 void
 Transducer::write(FILE *output, int const decalage)
 {
+  fwrite(HEADER_TRANSDUCER, 1, 4, output);
+
+  bool write_weights = weighted();
+
+  uint64_t features = 0;
+  if (write_weights) {
+      features |= TDF_WEIGHTS;
+  }
+  write_le(output, features);
+
   Compression::multibyte_write(initial, output);
   Compression::multibyte_write(finals.size(), output);
 
   int base = 0;
-  for(set<int>::iterator it = finals.begin(), limit = finals.end();
-      it != limit; it++)
+  for(auto& it : finals)
   {
-    Compression::multibyte_write(*it - base, output);
-    base = *it;
+    Compression::multibyte_write(it.first - base, output);
+    base = it.first;
+    if(write_weights)
+    {
+      Compression::long_multibyte_write(it.second, output);
+    }
   }
 
   base = transitions.size();
   Compression::multibyte_write(base, output);
-  for(map<int, multimap<int, int> >::iterator it = transitions.begin(),
-                                              limit = transitions.end();
-      it != limit; it++)
+  for(auto& it : transitions)
   {
-    Compression::multibyte_write(it->second.size(), output);
+    Compression::multibyte_write(it.second.size(), output);
     int tagbase = 0;
-    for(multimap<int, int>::iterator it2 = it->second.begin(),
-                                     limit2 = it->second.end();
-        it2 != limit2; it2++)
+    for(auto& it2 : it.second)
     {
-      Compression::multibyte_write(it2->first-tagbase+decalage, output);
-      tagbase = it2->first;
+      Compression::multibyte_write(it2.first - tagbase + decalage, output);
+      tagbase = it2.first;
 
-      if(it2->second >= it->first)
+      if(it2.second.first >= it.first)
       {
-        Compression::multibyte_write(it2->second-it->first, output);
+        Compression::multibyte_write(it2.second.first - it.first, output);
       }
       else
       {
-        Compression::multibyte_write(it2->second+base-it->first, output);
+        Compression::multibyte_write(it2.second.first + base - it.first, output);
+      }
+      if(write_weights)
+      {
+        Compression::long_multibyte_write(it2.second.second, output);
       }
     }
   }
@@ -537,16 +560,40 @@ Transducer::read(FILE *input, int const decalage)
 {
   Transducer new_t;
 
+  bool read_weights = false;
+
+  fpos_t pos;
+  if (fgetpos(input, &pos) == 0) {
+      char header[4]{};
+      fread(header, 1, 4, input);
+      if (strncmp(header, HEADER_TRANSDUCER, 4) == 0) {
+          auto features = read_le<uint64_t>(input);
+          if (features >= TDF_UNKNOWN) {
+              throw std::runtime_error("Transducer has features that are unknown to this version of lttoolbox - upgrade!");
+          }
+          read_weights = (features & TDF_WEIGHTS);
+      }
+      else {
+          // Old binary format
+          fsetpos(input, &pos);
+      }
+  }
+
   new_t.initial = Compression::multibyte_read(input);
   int finals_size = Compression::multibyte_read(input);
 
   int base = 0;
+  double base_weight = default_weight;
   while(finals_size > 0)
   {
     finals_size--;
 
     base += Compression::multibyte_read(input);
-    new_t.finals.insert(base);
+    if(read_weights)
+    {
+      base_weight = Compression::long_multibyte_read(input);
+    }
+    new_t.finals.insert(make_pair(base, base_weight));
   }
 
   base = Compression::multibyte_read(input);
@@ -561,11 +608,15 @@ Transducer::read(FILE *input, int const decalage)
       number_of_local_transitions--;
       tagbase += Compression::multibyte_read(input) - decalage;
       int state = (current_state + Compression::multibyte_read(input)) % base;
+      if(read_weights)
+      {
+        base_weight = Compression::long_multibyte_read(input);
+      }
       if(new_t.transitions.find(state) == new_t.transitions.end())
       {
         new_t.transitions[state].clear(); // force create
       }
-      new_t.transitions[current_state].insert(pair<int, int>(tagbase, state));
+      new_t.transitions[current_state].insert(make_pair(tagbase, make_pair(state, base_weight)));
     }
     number_of_states--;
     current_state++;
@@ -578,16 +629,16 @@ void
 Transducer::serialise(std::ostream &serialised) const
 {
   Serialiser<int>::serialise(initial, serialised);
-  Serialiser<set<int> >::serialise(finals, serialised);
-  Serialiser<map<int, multimap<int, int> > >::serialise(transitions, serialised);
+  Serialiser<map<int, double> >::serialise(finals, serialised);
+  Serialiser<map<int, multimap<int, pair<int, double> > > >::serialise(transitions, serialised);
 }
 
 void
 Transducer::deserialise(std::istream &serialised)
 {
   initial = Deserialiser<int>::deserialise(serialised);
-  finals = Deserialiser<set<int> >::deserialise(serialised);
-  transitions = Deserialiser<map<int, multimap<int, int> > >::deserialise(serialised);
+  finals = Deserialiser<map<int, double> >::deserialise(serialised);
+  transitions = Deserialiser<map<int, multimap<int, pair<int, double> > > >::deserialise(serialised);
 }
 
 void
@@ -608,63 +659,56 @@ Transducer::reverse(int const epsilon_tag)
 {
   joinFinals(epsilon_tag);
 
-  map<int, multimap<int, int> > temporal;
+  map<int, multimap<int, pair<int, double> > > tmp_transitions;
 
-  for(map<int, multimap<int, int> >::reverse_iterator it = transitions.rbegin(); it != transitions.rend(); it++)
+  for(map<int, multimap<int, pair<int, double> > >::reverse_iterator it = transitions.rbegin(); it != transitions.rend(); it++)
   {
-    multimap<int, int> aux = it->second;
+    auto aux = it->second;
     it->second.clear();
-    for(multimap<int, int>::iterator it2 = aux.begin(), limit2 = aux.end();
-        it2 != limit2; it2++)
+    for(auto& it2 : aux)
     {
-      if(it2->second >= it->first)
+      if(it2.second.first >= it->first)
       {
-        transitions[it2->second].insert(pair<int, int>(it2->first, it->first));
+        transitions[it2.second.first].insert(make_pair(it2.first, make_pair(it->first, it2.second.second)));
       }
       else
       {
-        temporal[it2->second].insert(pair<int, int>(it2->first, it->first));
+        tmp_transitions[it2.second.first].insert(make_pair(it2.first, make_pair(it->first, it2.second.second)));
       }
     }
-    if(temporal.find(it->first) != temporal.end())
+    if(tmp_transitions.find(it->first) != tmp_transitions.end())
     {
-      (it->second).insert(temporal[it->first].begin(), temporal[it->first].end());
-      temporal.erase(it->first);
+      it->second.insert(tmp_transitions[it->first].begin(), tmp_transitions[it->first].end());
+      tmp_transitions.erase(it->first);
     }
   }
 
-  for(map<int, multimap<int, int> >::reverse_iterator it = temporal.rbegin(),
-                                                      limit = temporal.rend();
+  for(map<int, multimap<int, pair<int, double> > >::reverse_iterator it = tmp_transitions.rbegin(),
+                                                                  limit = tmp_transitions.rend();
       it != limit; it++)
   {
-    for(multimap<int, int>::iterator it2 = it->second.begin(),
-                                     limit2 = it->second.end();
-	it2 != limit2; it2++)
+    for(auto& it2 : it->second)
     {
-      transitions[it->first].insert(pair<int, int>(it2->first, it2->second));
+      transitions[it->first].insert(make_pair(it2.first, it2.second));
     }
   }
 
   int tmp = initial;
-  initial = *(finals.begin());
+  initial = finals.begin()->first;
   finals.clear();
-  finals.insert(tmp);
+  finals.insert(make_pair(tmp, default_weight));
 }
 
 void
 Transducer::show(Alphabet const &alphabet, FILE *output, int const epsilon_tag) const
 {
-  map<int, multimap<int, int> > temporal;
-
-  for(map<int, multimap<int, int> >::const_iterator it = transitions.begin(); it != transitions.end(); it++)
+  for(auto& it : transitions)
   {
-    multimap<int, int> aux = it->second;
-
-    for(multimap<int, int>::const_iterator it2 = aux.begin(); it2 != aux.end(); it2++)
+    for(auto& it2 : it.second)
     {
-      pair<int, int> t = alphabet.decode(it2->first);
-      fwprintf(output, L"%d\t", it->first);
-      fwprintf(output, L"%d\t", it2->second);
+      auto t = alphabet.decode(it2.first);
+      fwprintf(output, L"%d\t", it.first);
+      fwprintf(output, L"%d\t", it2.second.first);
       wstring l = L"";
       alphabet.getSymbol(l, t.first);
       if(l == L"")  // If we find an epsilon
@@ -685,13 +729,15 @@ Transducer::show(Alphabet const &alphabet, FILE *output, int const epsilon_tag)
       {
         fwprintf(output, L"%S\t", r.c_str());
       }
+      fwprintf(output, L"%f\t", it2.second.second);
       fwprintf(output, L"\n");
     }
   }
 
-  for(set<int>::const_iterator it3 = finals.begin(); it3 != finals.end(); it3++)
+  for(auto& it3 : finals)
   {
-    fwprintf(output, L"%d\n", *it3);
+    fwprintf(output, L"%d\t", it3.first);
+    fwprintf(output, L"%f\n", it3.second);
   }
 }
 
@@ -699,42 +745,41 @@ int
 Transducer::getStateSize(int const state)
 {
  set<int> states;
- set<int> myclosure1 = closure(state, 0);
+ auto myclosure1 = closure(state, 0);
  states.insert(myclosure1.begin(), myclosure1.end());
  int num_transitions = 0;
 
- for(set<int>::iterator it2 = states.begin(); it2 != states.end(); it2++)
+ for(auto& it2 : states)
  {
-   num_transitions += transitions[*it2].size();
+   num_transitions += transitions[it2].size();
  }
 
  return num_transitions;
 }
 
 bool
-Transducer::recognise(wstring patro, Alphabet &a, FILE *err)
+Transducer::recognise(wstring pattern, Alphabet &a, FILE *err)
 {
   bool accepted = false;
-  set<int> states ;
+  set<int> states;
 
-  set<int> myclosure1 = closure(getInitial(), 0);
+  auto myclosure1 = closure(getInitial(), 0);
   states.insert(myclosure1.begin(), myclosure1.end());
   // For each of the characters in the input string
-  for(wstring::iterator it = patro.begin(); it != patro.end(); it++)
+  for(auto& it : pattern)
   {
     set<int> new_state;        //Transducer::closure(int const state, int const epsilon_tag)
-    int sym = *it;
     // For each of the current alive states
-    //fwprintf(err, L"step: %S %C (%d)\n", patro.c_str(), *it, sym);
-    for(set<int>::iterator it2 = states.begin(); it2 != states.end(); it2++)
+    //fwprintf(err, L"step: %S %C (%d)\n", pattern.c_str(), *it, sym);
+    for(auto& it2 : states)
     {
-      multimap<int, int> p = transitions[*it2];
+      auto& p = transitions[it2];
       // For each of the transitions in the state
 
-      for(multimap<int, int>::iterator it3 = p.begin(); it3 != p.end(); it3++)
+      for(auto& it3 : p)
       {
 
-	pair<int, int> t = a.decode(it3->first);
+        auto t = a.decode(it3.first);
         wstring l = L"";
         a.getSymbol(l, t.first);
         //wstring r = L"";
@@ -742,9 +787,9 @@ Transducer::recognise(wstring patro, Alphabet &a, FILE *err)
 
         //fwprintf(err, L"  -> state: %d, trans: %S:%S, targ: %d\n", *it2, (l == L"") ?  L"ε" : l.c_str(),  (r == L"") ?  L"ε" : r.c_str(), it3->second);
         //if(l.find(*it) != wstring::npos || l == L"" )
-        if(l.find(*it) != wstring::npos)
+        if(l.find(it) != wstring::npos)
         {
-          set<int> myclosure = closure(it3->second, 0);
+          auto myclosure = closure(it3.second.first, 0);
           //wcerr << L"Before closure alives: " <<new_state.size() << endl;
           new_state.insert(myclosure.begin(), myclosure.end());
           //wcerr << L"After closure alives: " <<new_state.size() << endl;
@@ -753,9 +798,9 @@ Transducer::recognise(wstring patro, Alphabet &a, FILE *err)
     }
     states = new_state;
   }
-  for(set<int>::iterator it4 = states.begin(); it4 != states.end(); it4++)
+  for(auto& it4 : states)
   {
-    if(isFinal(*it4))
+    if(isFinal(it4))
     {
       accepted = true;
     }
@@ -766,10 +811,10 @@ Transducer::recognise(wstring patro, Alphabet &a, FILE *err)
 
 void
 Transducer::unionWith(Alphabet &my_a,
-  Transducer &t,
-  int const epsilon_tag)
+                      Transducer &t,
+                      int const epsilon_tag)
 {
-  finals.insert(insertTransducer(initial, t, epsilon_tag));
+  finals.insert(make_pair(insertTransducer(initial, t, epsilon_tag), default_weight));
 }
 
 Transducer
@@ -777,19 +822,13 @@ Transducer::appendDotStar(set<int> const &loopback_symbols, int const epsilon_ta
 {
   Transducer prefix_transducer(*this);
 
-  for(set<int>::iterator prefix_it = prefix_transducer.finals.begin(),
-                           prefix_limit = prefix_transducer.finals.end();
-      prefix_it != prefix_limit;
-      prefix_it++)
+  for(auto& prefix_it : prefix_transducer.finals)
   {
-    for(set<int>::iterator loopback_it = loopback_symbols.begin(),
-                           loopback_limit = loopback_symbols.end();
-    loopback_it != loopback_limit;
-    loopback_it++)
+    for(auto& loopback_it : loopback_symbols)
     {
-      if((*loopback_it) != epsilon_tag) // TODO: Necessary? Minimization should remove epsilon loopbacks anyway
+      if(loopback_it != epsilon_tag) // TODO: Necessary? Minimization should remove epsilon loopbacks anyway
       {
-        prefix_transducer.linkStates(*prefix_it, *prefix_it, *loopback_it);
+        prefix_transducer.linkStates(prefix_it.first, prefix_it.first, loopback_it, prefix_it.second);
       }
     }
   }
@@ -826,12 +865,10 @@ Transducer::copyWithTagsFirst(int start,
     seen.insert(current);
     int this_src = current.first, this_lemqlast = current.second;
 
-    for(multimap<int, int>::iterator trans_it = transitions[this_src].begin(),
-          trans_limit = transitions[this_src].end();
-        trans_it != trans_limit;
-        trans_it++)
+    for(auto& trans_it : transitions[this_src])
     {
-      int label = trans_it->first, this_trg = trans_it->second;
+      int label = trans_it.first, this_trg = trans_it.second.first;
+      double this_wt = trans_it.second.second;
       int left_symbol = alphabet.decode(label).first;
 
       // Anything after the first tag goes before the lemq, whether
@@ -846,7 +883,7 @@ Transducer::copyWithTagsFirst(int start,
         {
           // We've reached the first tag
           new_src = states_this_new[start];
-          lemq.finals.insert(this_lemqlast);
+          lemq.finals.insert(make_pair(this_lemqlast, default_weight));
         }
         else
         {
@@ -862,7 +899,7 @@ Transducer::copyWithTagsFirst(int start,
           states_this_new.insert(make_pair(this_trg, new_t.newState()));
         }
         int new_trg = states_this_new[this_trg];
-        new_t.linkStates(new_src, new_trg, label);
+        new_t.linkStates(new_src, new_trg, label, this_wt);
 
         if(isFinal(this_src))
         {
@@ -883,7 +920,7 @@ Transducer::copyWithTagsFirst(int start,
           states_this_lemq.insert(make_pair(this_trg, lemq.newState()));
         }
         int lemq_trg = states_this_lemq[this_trg];
-        lemq.linkStates(lemq_src, lemq_trg, label);
+        lemq.linkStates(lemq_src, lemq_trg, label, this_wt);
         if(seen.find(make_pair(this_trg, this_trg)) == seen.end())
         {
           todo.push_front(make_pair(this_trg, this_trg));
@@ -892,26 +929,21 @@ Transducer::copyWithTagsFirst(int start,
     } // end for transitions
   } // end while todo
 
-  for(set<SearchState>::iterator it = finally.begin(), limit = finally.end();
-      it != limit;
-      it++)
+  for(auto& it : finally)
   {
-    int last_tag = it->first,
-      this_lemqlast = it->second;
+    int last_tag = it.first,
+      this_lemqlast = it.second;
     // copy lemq, letting this_lemqlast be the only final state in newlemq
     Transducer newlemq = Transducer(lemq);
     newlemq.finals.clear();
-    newlemq.finals.insert(states_this_lemq[this_lemqlast]);
+    newlemq.finals.insert(make_pair(states_this_lemq[this_lemqlast], default_weight));
     newlemq.minimize();
 
     int group_start = new_t.newState();
-    new_t.linkStates(states_this_new[last_tag], group_start, group_label);
+    new_t.linkStates(states_this_new[last_tag], group_start, group_label, default_weight);
 
     // append newlemq into the group after last_tag:
-    new_t.finals.insert(
-      new_t.insertTransducer(group_start,
-                             newlemq)
-      );
+    new_t.finals.insert(make_pair(new_t.insertTransducer(group_start, newlemq), default_weight));
   }
 
   return new_t;
@@ -939,13 +971,10 @@ Transducer::moveLemqsLast(Alphabet const &alphabet,
     int this_src = todo.front();
     todo.pop_front();
     seen.insert(this_src);
-    for(multimap<int, int>::iterator trans_it = transitions[this_src].begin(),
-          trans_limit = transitions[this_src].end();
-        trans_it != trans_limit;
-        trans_it++)
+    for(auto& trans_it : transitions[this_src])
     {
-      int label = trans_it->first,
-       this_trg = trans_it->second;
+      int label = trans_it.first,
+       this_trg = trans_it.second.first;
       wstring left = L"";
       alphabet.getSymbol(left, alphabet.decode(label).first);
       int new_src = states_this_new[this_src];
@@ -953,9 +982,9 @@ Transducer::moveLemqsLast(Alphabet const &alphabet,
       if(left == COMPILER_GROUP_ELEM)
       {
         Transducer tagsFirst = copyWithTagsFirst(this_trg, label, alphabet, epsilon_tag);
-        new_t.finals.insert(
-          new_t.insertTransducer(new_src, tagsFirst, epsilon_tag)
-          );
+        new_t.finals.insert(make_pair(
+          new_t.insertTransducer(new_src, tagsFirst, epsilon_tag), default_weight
+          ));
       }
       else
       {
@@ -964,7 +993,7 @@ Transducer::moveLemqsLast(Alphabet const &alphabet,
           states_this_new.insert(make_pair(this_trg, new_t.newState()));
         }
         int new_trg = states_this_new[this_trg];
-        new_t.linkStates(new_src, new_trg, label);
+        new_t.linkStates(new_src, new_trg, label, default_weight);
         if(seen.find(this_trg) == seen.end())
         {
           todo.push_front(this_trg);
@@ -973,11 +1002,9 @@ Transducer::moveLemqsLast(Alphabet const &alphabet,
     }
   }
 
-  for(set<int>::iterator it = finals.begin(), limit = finals.end();
-      it != limit;
-      it++)
+  for(auto& it : finals)
    {
-     new_t.finals.insert(states_this_new[*it]);
+     new_t.finals.insert(make_pair(states_this_new[it.first], it.second));
    }
 
   return new_t;
@@ -986,9 +1013,9 @@ Transducer::moveLemqsLast(Alphabet const &alphabet,
 
 Transducer
 Transducer::intersect(Transducer &trimmer,
-  Alphabet const &this_a,
-  Alphabet const &trimmer_a,
-  int const epsilon_tag)
+                      Alphabet const &this_a,
+                      Alphabet const &trimmer_a,
+                      int const epsilon_tag)
 {
   joinFinals(epsilon_tag);
   /**
@@ -1040,12 +1067,10 @@ Transducer::intersect(Transducer &trimmer,
     int trimmed_src = states_this_trimmed[current];
 
     // First loop through _epsilon_ transitions of trimmer
-    for(multimap<int, int>::iterator trimmer_trans_it = trimmer.transitions.at(trimmer_src).begin(),
-          trimmer_trans_limit = trimmer.transitions.at(trimmer_src).end();
-        trimmer_trans_it != trimmer_trans_limit;
-        trimmer_trans_it++) {
-      int trimmer_label = trimmer_trans_it->first,
-          trimmer_trg   = trimmer_trans_it->second;
+    for(auto& trimmer_trans_it : trimmer.transitions.at(trimmer_src)) {
+      int trimmer_label = trimmer_trans_it.first,
+          trimmer_trg   = trimmer_trans_it.second.first;
+      double trimmer_wt = trimmer_trans_it.second.second;
       wstring trimmer_left = L"";
       trimmer_a.getSymbol(trimmer_left, trimmer_a.decode(trimmer_label).first);
 
@@ -1065,19 +1090,18 @@ Transducer::intersect(Transducer &trimmer,
         int trimmed_trg = states_this_trimmed[next];
         trimmed.linkStates(trimmed_src,
                            trimmed_trg,
-                           epsilon_tag);
+                           epsilon_tag,
+                           trimmer_wt);
       }
     }
 
     // Loop through arcs from this_src; when our arc matches an arc
     // from live_trimmer_states, add that to (the front of) todo:
-    for(multimap<int, int>::iterator trans_it = transitions[this_src].begin(),
-                                     trans_limit = transitions[this_src].end();
-        trans_it != trans_limit;
-        trans_it++)
+    for(auto& trans_it : transitions[this_src])
     {
-      int this_label = trans_it->first,
-          this_trg   = trans_it->second;
+      int this_label = trans_it.first,
+          this_trg   = trans_it.second.first;
+      double this_wt = trans_it.second.second;
       wstring this_right = L"";
       this_a.getSymbol(this_right, this_a.decode(this_label).second);
 
@@ -1100,7 +1124,8 @@ Transducer::intersect(Transducer &trimmer,
         int trimmed_trg = states_this_trimmed[next];
         trimmed.linkStates(trimmed_src, // fromState
                            trimmed_trg, // toState
-                           this_label); // symbol-pair, using this alphabet
+                           this_label, // symbol-pair, using this alphabet
+                           this_wt); //weight of transduction
       }
       else if ( this_right == compoundOnlyLSymbol
                 || this_right == compoundRSymbol
@@ -1126,7 +1151,8 @@ Transducer::intersect(Transducer &trimmer,
         int trimmed_trg = states_this_trimmed[next];
         trimmed.linkStates(trimmed_src, // fromState
                            trimmed_trg, // toState
-                           this_label); // symbol-pair, using this alphabet
+                           this_label, // symbol-pair, using this alphabet
+                           this_wt); //weight of transduction
       }
       else
       {
@@ -1141,13 +1167,10 @@ Transducer::intersect(Transducer &trimmer,
           trimmer_src = trimmer_preplus;
         }
 
-        for(multimap<int, int>::iterator trimmer_trans_it = trimmer.transitions.at(trimmer_src).begin(),
-              trimmer_trans_limit = trimmer.transitions.at(trimmer_src).end();
-            trimmer_trans_it != trimmer_trans_limit;
-            trimmer_trans_it++)
+        for(auto& trimmer_trans_it : trimmer.transitions.at(trimmer_src))
         {
-          int trimmer_label = trimmer_trans_it->first,
-              trimmer_trg   = trimmer_trans_it->second;
+          int trimmer_label = trimmer_trans_it.first,
+              trimmer_trg   = trimmer_trans_it.second.first;
           wstring trimmer_left = L"";
           trimmer_a.getSymbol(trimmer_left, trimmer_a.decode(trimmer_label).first);
 
@@ -1170,24 +1193,22 @@ Transducer::intersect(Transducer &trimmer,
             int trimmed_trg = states_this_trimmed[next];
             trimmed.linkStates(trimmed_src, // fromState
                                trimmed_trg, // toState
-                               this_label); // symbol-pair, using this alphabet
+                               this_label, // symbol-pair, using this alphabet
+                               this_wt); //weight of transduction
           }
         } // end loop arcs from trimmer_src
       } // end if JOIN else
     } // end loop arcs from this_src
   } // end while todo
 
-  for(map<SearchState, int >::iterator it = states_this_trimmed.begin(),
-        limit = states_this_trimmed.end();
-      it != limit;
-      it++)
+  for(auto& it : states_this_trimmed)
   {
-    int s_this = it->first.first;
-    int s_trimmer = it->first.second.first; // ignore the preplus here
-    int s_trimmed = it->second;
+    int s_this = it.first.first;
+    int s_trimmer = it.first.second.first; // ignore the preplus here
+    int s_trimmed = it.second;
     if(isFinal(s_this) && trimmer.isFinal(s_trimmer))
     {
-      trimmed.finals.insert(s_trimmed);
+      trimmed.finals.insert(make_pair(s_trimmed, default_weight));
     }
   }
 
