@@ -60,13 +60,13 @@ class PrintWeights(unittest.TestCase, ProcTest):
     procdix = "data/cat-weight.att"
     procflags = ["-W"]
     inputs = ["cat"]
-    expectedOutputs = ["^cat/cat+n<W:11.002748>/cat+v<W:12.034480>$"]
+    expectedOutputs = ["^cat/cat+n<W:11.528235>/cat+v<W:12.559967>$"]
 
 class PrintWeightsNegative(unittest.TestCase, ProcTest):
     procdix = "data/cat-weight-negative.att"
     procflags = ["-W"]
     inputs = ["cat"]
-    expectedOutputs = ["^cat/cat+n<W:9.353620>/cat+v<W:10.385352>$"]
+    expectedOutputs = ["^cat/cat+n<W:8.828133>/cat+v<W:9.859865>$"]
 
 class PrintNAnalyses(unittest.TestCase, ProcTest):
     procdix = "data/cat-weight.att"
@@ -85,5 +85,50 @@ class AllEntryWeights(unittest.TestCase, ProcTest):
     procflags = ["-W"]
     inputs = ["nanow"]
     expectedOutputs = ["^nanow/nan<n><ma><du><gen><W:32.120000>/nan<n><ma><du><acc><W:34.120000>/nan<n><ma><pl><gen><W:39.120000>/nan<n><ma><pl><acc><W:41.120000>$"]
+
+class Intergeneration(unittest.TestCase, ProcTest):
+    procdix = "data/intergen.dix"
+    procflags = ["-x"]
+    inputs = ["la dona ~dóna tot"]
+    expectedOutputs = ["la dona dona tot"]
+
+class GardenPathMwe(unittest.TestCase, ProcTest):
+    procdix = "data/gardenpath-mwe.dix"
+    inputs          = ["x[ <br/> ]opp.",
+                        "legge opp[<br/>]x.",
+                       "y[A]x",
+                        "[A]y x",
+                        "legge opp[<br/>]",
+                        "legge[][\n]L[<\/p>\n]",
+                       ]
+    expectedOutputs = ["^x/*x$[ <br/> ]^opp/opp<pr>$^./.<sent>$",
+                        "^legge/legge<vblex><inf>$ ^opp/opp<pr>$[<br/>]^x/*x$^./.<sent>$",
+                       "^y/*y$[A]^x/*x$",
+                        "[A]^y/*y$ ^x/*x$",
+                        "^legge/legge<vblex><inf>$ ^opp/opp<pr>$[<br/>]",
+                        "^legge/legge<vblex><inf>$[][\n]^L/*L$[<\/p>\n]",
+                       ]
+
+class GardenPathMweNewlines(unittest.TestCase, ProcTest):
+    procdix = "data/gardenpath-mwe.dix"
+    inputs          = ["""St.[
+]Petersburg[
+]X y.[][
+
+]F G.[][
+]"""]
+    expectedOutputs = ["""^St. Petersburg/St. Petersburg<np>$[
+][
+]^X y/X y<np>$^./.<sent>$[][
+
+]^F/F<np>$ ^G/G<np>$^./.<sent>$[][
+]"""
+                       ]
+
+class CatMultipleFstsTransducer(unittest.TestCase, ProcTest):
+    procdix = "data/cat-multiple-fst.att"
+    inputs = ["cat", "cats"]
+    expectedOutputs = ["^cat/cat+n/cat+v$", "^cats/cat+n+<pl>$"]
+
 # These fail on some systems:
 #from null_flush_invalid_stream_format import *
